// file      : xsde/cxx/parser/state.ixx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <assert.h>

namespace xsde
{
  namespace cxx
  {
    namespace parser
    {
      // stack
      //

      inline stack::
      ~stack ()
      {
        delete[] data_;
      }

      inline stack::
      stack (size_t el_size, void* first_el)
          : el_size_ (el_size), first_ (first_el),
            data_ (0), size_ (0), capacity_ (0)
      {
      }

      inline void stack::
      pop ()
      {
        --size_;
      }

      inline stack::error stack::
      push ()
      {
        if (size_ > capacity_)
          if (error e = grow ())
            return e;

        ++size_;

        return error_none;
      }

      inline void* stack::
      top ()
      {
        return size_ == 1 ? first_ : data_ + (size_ - 1) * el_size_;
      }

      inline void* stack::
      under_top ()
      {
        return size_ == 2 ? first_ : data_ + (size_ - 2) * el_size_;
      }

      inline size_t stack::
      element_size () const
      {
        return el_size_;
      }


      // parse_stack
      //
      inline parse_stack::
      parse_stack ()
          : stack_ (sizeof (parse_state), &first_)
      {
      }

      inline stack::error parse_stack::
      push ()
      {
        if (stack::error e = stack_.push ())
          return e;

        parse_state* s = static_cast<parse_state*> (stack_.top ());

        s->any_ = false;
        s->depth_ = 0;
        s->parser_ = 0;

        return stack::error_none;
      }

      inline void parse_stack::
      pop ()
      {
        stack_.pop ();
      }

      inline parse_state& parse_stack::
      top ()
      {
        return *static_cast<parse_state*> (stack_.top ());
      }

      inline parse_state& parse_stack::
      under_top ()
      {
        return *static_cast<parse_state*> (stack_.under_top ());
      }
    }
  }
}

